{{ SumoDrive.spin }}

''   Controls 2 DC Motor PWMs using the L293D H-Bridge at 5kHz.
''   Controls 4 hobby servo PWMs with 1-2msec pulse with and 16msec pulse repetition rate.

''   Use SetMotor to assign a new speed from -100 to 100 for one or more motors
''   Use SetServo to assign a new position from 0 to 100 for one or more servos

''   Setting a motor or servo to 0 will disable the channel.


CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 6_000_000

VAR
  long cog                     'cog flag/id
      
PUB start(buff_ptr) : okay
'' Start the motor driver - starts a cog
'' returns false if no cog available
  stop
  okay := cog := cognew(@entry, buff_ptr) + 1

PUB stop

'' Stop serial driver - frees a cog

  if cog
    cogstop(cog~ - 1)

DAT

entry         org       0

{ Initialization }
              mov       p0,         par
              rdlong    speed_buff, p0              'Pointer to Speed Buffer

              
              andn      outa,       m0_dir0
              andn      outa,       m1_dir0
              andn      outa,       m0_dir1
              andn      outa,       m1_dir1
              andn      outa,       m0_pwm
              andn      outa,       m1_pwm
              andn      outa,       s0_pwm
              andn      outa,       s1_pwm
              andn      outa,       s2_pwm
              andn      outa,       s3_pwm

              or        dira,       m0_dir0
              or        dira,       m1_dir0
              or        dira,       m0_dir1
              or        dira,       m1_dir1
              or        dira,       m0_pwm
              or        dira,       m1_pwm
              or        dira,       s0_pwm
              or        dira,       s1_pwm
              or        dira,       s2_pwm
              or        dira,       s3_pwm

              mov       m0_en,      #0          ' Disable all outputs
              mov       m1_en,      #0
              mov       s0_en,      #0
              mov       s1_en,      #0
              mov       s2_en,      #0
              mov       s3_en,      #0              

              mov       ctra,       CounterMode     ' Setup counter to time pwm cycles
              mov       ctrb,       CounterMode
              
              mov       frqa,       MotorFrq
              mov       frqb,       ServoFrq

              mov       phsa,       #0
              mov       phsb,       #0

              mov       c0,         cnt
              add       c0,         MotorPeriod

                            
{ Start Main Loop }
MainLoop              
            
UpdatePWM
              mov       p0,         par
              
              rdlong    changed,    p0        wz    ' Check if speed commands have changed
  '      if_z  jmp       #ResetCounters              ' Only update if a change flag set

              
              add       p0,         #4              ' Read new control values
              rdlong    motor_mode, p0
              add       p0,         #4
              rdlong    servo_mode, p0
              add       p0,         #4
              rdlong    MotorPeriod, p0
              add       p0,         #4
              rdlong    m0_speed,   p0
              add       p0,         #4
              rdlong    m1_speed,   p0

              add       p0,         #4
              rdlong    s0_pos,     p0
              add       p0,         #4
              rdlong    s1_pos,     p0
              add       p0,         #4
              rdlong    s2_pos,     p0
              add       p0,         #4
              rdlong    s3_pos,     p0

              
:motor0       mov       m0_en,      m0_pwm           ' Update Motor Control Parameters

              test      motor_mode, #%0000_0001 wz
        if_nz jmp       #:m0dir
              mov       m0_en,      #0
              test      motor_mode, #%0000_0010 wz
              muxz      outa,       m0_dir0
              muxz      outa,       m0_dir1
              jmp       #:motor1
              
:m0dir        test      motor_mode, #%0000_0100 wz
              muxz      outa,       m0_dir0
              muxnz     outa,       m0_dir1                

:motor1       mov       m1_en,      m1_pwm
              test      motor_mode, #%0001_0000 wz
        if_nz jmp       #:m1dir
              mov       m0_en,      #0
              test      motor_mode, #%0010_0000 wz
              muxz      outa,       m1_dir0
              muxz      outa,       m1_dir1
              jmp       #:servos
              
:m1dir        test      motor_mode, #%0100_0000 wz
              muxz      outa,       m1_dir0
              muxnz     outa,       m1_dir1                

:servos
              mov       s0_en,      s0_pwm
              mov       s1_en,      s1_pwm
              mov       s2_en,      s2_pwm
              mov       s3_en,      s3_pwm
              
              test      servo_mode, #%0000_0001 wz
        if_z  mov       s0_en,      #0
              test      servo_mode, #%0000_0010 wz
        if_z  mov       s1_en,      #0
              test      servo_mode, #%0000_0100 wz
        if_z  mov       s2_en,      #0
              test      servo_mode, #%0000_1000 wz
        if_z  mov       s3_en,      #0

:done

              mov       changed,    #0                     ' Update changed flag
              mov       p0,         par
              wrlong    changed,    p0
              
ResetCounters
              cmp       ServoPeriod,  phsb wc, wz         ' Check for servo period reset
        if_a  jmp       #:servo0

              mov       servo_done,   #0
              mov       servo_enable, #0
              mov       servo_stop,   ServoPeriod
              mov       phsb,         #0
              
              
:servo0       test      servo_done,   #%0001 wz
        if_nz jmp       #:servo1    
              cmp       s0_start,     phsb wc, wz
        if_a  jmp       #:motors
              mov       servo_enable, s0_en
              or        outa,         s0_en
       
              
              mov       servo_stop,   phsb
              add       servo_stop,   s0_pos wr
              
              or        servo_done,   #%0001
              jmp       #:motors
              
:servo1       test      servo_done,   #%0010 wz
        if_nz jmp       #:servo2    
              cmp       s1_start,     phsb wc, wz
        if_a  jmp       #:motors
              mov       servo_enable, s1_en
              or        outa,         s1_en
              mov       servo_stop,   phsb
              add       servo_stop,   s1_pos wr
              or        servo_done,   #%0010
              jmp       #:motors

:servo2       test      servo_done,   #%0100 wz
        if_nz jmp       #:servo3    
              cmp       s2_start,     phsb wc, wz
        if_a  jmp       #:motors
              mov       servo_enable, s2_en
              or        outa,         s2_en
              mov       servo_stop,   phsb
              add       servo_stop,   s2_pos wr
              or        servo_done,   #%0100
              jmp       #:motors

:servo3       test      servo_done,   #%1000 wz
        if_nz jmp       #:motors   
              cmp       s3_start,     phsb wc, wz
        if_a  jmp       #:motors
              mov       servo_enable, s3_en
              or        outa,         s3_en
              mov       servo_stop,   phsb
              add       servo_stop,   s3_pos wr
              or        servo_done,   #%1000
              jmp       #:motors
                                  
                                  
:motors       or        outa,       m0_en             ' Initially set pwm outputs high if enabled
              or        outa,       m1_en
              mov       phsa,       #0
        
GeneratePWM
                    
:motors       cmp       m0_speed,   phsa wc, wz
        if_be andn      outa,       m0_en
              
              cmp       m1_speed,   phsa wc, wz
        if_be andn      outa,       m1_en
             
:servos
              cmp       servo_stop, phsb wc, wz     ' Check if the current servo has expired
        if_be andn      outa,       servo_enable

                    
              cmp       MotorPeriod, phsa wc, wz
        if_be jmp       #MainLoop
              jmp       #GeneratePWM
 
'Initialized Variables
CounterMode   long      %0_11111_000_00000000_000000_000_000000


MotorFrq      long      1
MotorPeriod   long      96*800-9     '200uSec - 5kHz

SpeedMax      long      96*800
SpeedMin      long      46

SpeedTest     long      96*100

ServoFrq      long      1             'Servo timer 10usec resolution
ServoPeriod   long      96_000 * 16   '16msec

PosOffset     long      960 * 58      '.55 msec pulse width for 0 degree
PosScale      long      1140 '1150    '12.29 uSec / degree 

'Pin Assignments
m0_pwm        long      1 << 14 
m0_dir0       long      1 << 13 
m0_dir1       long      1 << 15


m1_pwm        long      1 << 10
m1_dir0       long      1 << 11
m1_dir1       long      1 << 12


s0_pwm        long      1 << 4
s1_pwm        long      1 << 5
s2_pwm        long      1 << 6
s3_pwm        long      1 << 7


'Servo Start Offset
s0_start      long      960 * 0
s1_start      long      960 * 400
s2_start      long      960 * 800
s3_start      long      960 * 1200

'Uninitialized Varials
spd           res       1
pos           res       1

'Temporary Variables
p0            res       1

t0            res       1
t1            res       1
t2            res       1

x             res       1
y             res       1

changed       res       1

c0            res       1

'State Variables
window        res       1           'holds counter value of next window
speed_buff    res       1           'pointer to speed buffer (mtrs+1 longs)
done_flag     res       1

m0_speed      res       1           'motor 1 speed
m1_speed      res       1           'motor 2 speed

m0_target     res       1
m1_target     res       1

m0_en         res       1           'motor 1 enable (state machine)
m1_en         res       1           'motor 2 enable (state machine)

s0_en         res       1           'servo 1 enable
s1_en         res       1           'servo 2 enable
s2_en         res       1           'servo 3 enable
s3_en         res       1           'servo 4 enable

servo_done    res       1
servo_stop    res       1
servo_enable  res       1

s0_pos        res       1           'servo 1 position
s1_pos        res       1           'servo 2 position
s2_pos        res       1           'servo 3 position
s3_pos        res       1           'servo 4 position

motor_mode    res       1
servo_mode    res       1