{{ SumoBotMain.spin  }}

'' This object is used to initialize and encapsulate the low level drivers
'' It should only be included once for each project, while the implementing
'' code in SumoBot.spin can be included multiple times throughout the project


CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 6_000_000

  DataAddress    = $8000
  ConfigAddress  = $9000
  ProgramAddress = $A000
  
  PageSize = 32
  
VAR
  long  cog

  long data_begin
  long data_end
  long config_begin
  long config_end

  long program_begin
  long program_end
  
OBJ
  mouse :  "SumoMouse"
  drive :  "SumoDrive"
  sonar :  "SumoSonar"
  spi :    "SumoIO"
  sumo :   "SumoBot"
  eeprom : "eeprom"
  
  
PUB Init | i
  ' Initialize the config and data sections 
  sumo.GetDataBlock(@data_begin, @data_end)
  sumo.GetConfigBlock(@config_begin, @config_end)

  eeprom.Initialize(eeprom#BootPin) 
   
  {{if byte[data_begin] == 0
    LoadDataBlock
  else
    SaveDataBlock

  if byte[config_begin] == 0
    LoadConfigBlock
  else
    SaveConfigBlock}}
    
  ' Start Low Level Drivers
  mouse.start(sumo.GetDataPtr(sumo#MOUSE))
  sonar.start(sumo.GetDataPtr(sumo#SONAR))
  drive.start(sumo.GetDataPtr(sumo#DRIVE))
  spi.start(sumo.GetDataPtr(sumo#ANALOG))    


  ' Finish robot initialization
  sumo.Init
  
{{
   EEPROM Usage

   Non-volatile settings are stored in two reserved portions of the EEPROM just above the main program
   data space. One block is used to store the state of variables directly used by the low level drivers.
   This data block is relatively volatile, however, eeprom configuration of certain parameters may be usefull.
   
   The second non-volatile storage block is used for configuration and calibration parameters that must be
   stored in the EEPROM. Two different blocks are used to allow changes in the size/layout of one block to
   not effect the other.
   
}}

'' Save all data from DATA_BEGIN to DATA_END into non-volatile portion of EEPROM 
PUB SaveDataBlock 
   eeprom.WriteBlock(data_begin, DataAddress, data_end - data_begin + 1)
  
'' Restore settings from EEPROM to initialize data from DATA_BEGIN to DATA_END
PUB LoadDataBlock
   eeprom.ReadBlock(data_begin, DataAddress, data_end - data_begin + 1) 

'' Save all data from CONFIG_BEGIN to CONFIG_END into non-volatile portion of EEPROM 
PUB SaveConfigBlock
   eeprom.WriteBlock(config_begin, ConfigAddress, config_end - config_begin + 1)     

'' Restore settings from EEPROM to initialize data from CONFIG_BEGIN to CONFIG_END
PUB LoadConfigBlock
   eeprom.ReadBlock(config_begin, ConfigAddress, config_end - config_begin + 1)